<?php

namespace Drupal\instagram_sync\Form;

use Drupal\instagram_sync\APITypeInterface;
use Drupal\instagram_sync\ApiTypePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Instagram Sync settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * API Type plugin manager.
   *
   * @var \Drupal\instagram_sync\ApiTypePluginManager
   */
  protected ApiTypePluginManager $apiTypePluginManager;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'instagram_sync_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      INSTAGRAM_SYNC_CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.api_type'),
    );
  }

  /**
   * Constructs a SettingsForm.
   *
   * @param \Drupal\instagram_sync\ApiTypePluginManager $api_type_plugin_manager
   *   Api Type Plugin manager.
   */
  public function __construct(ApiTypePluginManager $api_type_plugin_manager) {
    $this->apiTypePluginManager = $api_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(INSTAGRAM_SYNC_CONFIG_NAME);

    $options = [];
    $pluginForms = [];
    foreach ($this->apiTypePluginManager->getDefinitions() as $plugin_id => $definition) {
      $options[$plugin_id] = $definition['label'];

      /** @var \Drupal\instagram_sync\APITypeInterface $apiPlugin */
      $apiPlugin = $this->apiTypePluginManager->createInstance($plugin_id, $definition);
      $pluginForms[$plugin_id] = $apiPlugin->buildConfigurationForm($form, $form_state);
    }

    $form['wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'settings-form-wrapper'],
    ];

    $form['api_type_plugin'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => $this->t('Choose API Type plugin'),
      '#required' => TRUE,
      '#default_value' => $config->get('api_type_plugin') ?? NULL,
    ];

    $form['plugins'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Plugins configuration'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    ];

    foreach ($pluginForms as $pluginId => $pluginForm) {
      $form['plugins'][$pluginId] = $pluginForm;
    }

    $form['settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    ];

    $form['settings']['video_ignore'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ignore video download on import.'),
      '#default_value' => $config->get('settings.video_ignore'),
    ];

    $form['settings']['carousel_ignore'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ignore pictures for carousels, except first.'),
      '#description' => $this->t('Loads just first image, instead of whole carousel.'),
      '#default_value' => $config->get('settings.carousel_ignore'),
    ];

    $form['settings']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Quantity limit of posts to import'),
      '#description' => $this->t('Please enter an integer value. Leave empty to import all posts.'),
      '#required' => TRUE,
      '#min' => 3,
      '#step' => 1,
      '#default_value' => $config->get('settings.limit') ?? '',
    ];

    $form['token_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Refresh settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    ];

    $form['token_settings']['refresh_auto_update'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable access token auto-refresh'),
      '#default_value' => $config->get('token_settings.refresh_auto_update'),
    ];

    $form['token_settings']['refresh_period'] = [
      '#type' => 'number',
      '#title' => $this->t('Token refresh period'),
      '#description' => $this->t('Set the token refresh period in days. Maximum: 59 days.'),
      '#states' => [
        'visible' => [
          ':input[name="refresh_auto_update"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="refresh_auto_update"]' => ['checked' => TRUE],
        ],
      ],
      '#default_value' => $config->get('token_settings.refresh_period'),
      '#min' => 1,
      '#max' => 59,
      '#step' => 1,
    ];

    $form['cron_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('CRON settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
    ];

    $form['cron_settings']['auto_update'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable auto-sync'),
      '#default_value' => $config->get('cron_settings.auto_update'),
    ];

    $form['cron_settings']['refresh_period'] = [
      '#type' => 'number',
      '#title' => $this->t('Auto-sync period'),
      '#description' => $this->t('Set the auto-sync period in days. Maximum: 30 days.'),
      '#states' => [
        'visible' => [
          ':input[name="auto_update"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="auto_update"]' => ['checked' => TRUE],
        ],
      ],
      '#default_value' => $config->get('cron_settings.refresh_period'),
      '#min' => 1,
      '#max' => 30,
      '#step' => 1,
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['instagram_import'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import instagram posts'),
      '#submit' => ['::processBatch'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $pluginId = $form_state->getValue('api_type_plugin');

    $apiPlugin = $this->apiTypePluginManager->createInstance($pluginId);
    if ($apiPlugin instanceof APITypeInterface) {
      $apiPlugin->validateConfigurationForm($form, $form_state);
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config(INSTAGRAM_SYNC_CONFIG_NAME)
      ->set('api_type_plugin', $form_state->getValue('api_type_plugin'))
      ->set('plugins', $form_state->getValue('plugins'))
      ->set('token_settings', $form_state->getValue('token_settings'))
      ->set('settings', $form_state->getValue('settings'))
      ->set('cron_settings', $form_state->getValue('cron_settings'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Starts batch import process on button click.
   */
  public function processBatch(array &$form, FormStateInterface &$form_state) {
    $plugin = $form_state->getValue('api_type_plugin');
    $postLimit = $form_state->getValue('settings')['limit'];
    instagram_sync_request_batch($plugin, $postLimit);
  }

}
