<?php

namespace Drupal\instagram_sync;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list controller for the instagram sync entity type.
 */
class InstagramSyncListBuilder extends EntityListBuilder {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected DateFormatterInterface $dateFormatter;

  /**
   * Image style storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   *   Image style storage variable.
   */
  protected EntityStorageInterface $imageStyleStorage;

  /**
   * Constructs a new InstagramSyncListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager class.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeInterface $entity_type, EntityTypeManagerInterface $entityTypeManager, DateFormatterInterface $date_formatter) {
    parent::__construct($entity_type, $entityTypeManager->getStorage($entity_type->id()));
    $this->dateFormatter = $date_formatter;
    $this->imageStyleStorage = $entityTypeManager->getStorage('image_style');
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build['table'] = parent::render();

    $total = $this->getStorage()
      ->getQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();

    $build['summary']['#markup'] = $this->t('Total instagram syncs: @total', ['@total' => $total]);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('ID');
    $header['type'] = $this->t('Type');
    $header['image'] = $this->t('Image Preview');
    $header['caption'] = $this->t('Caption');
    $header['like_count'] = $this->t('Like Count');
    $header['comments_count'] = $this->t('Comments Count');
    $header['created'] = $this->t('Created');
    $header['changed'] = $this->t('Updated');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\instagram_sync\InstagramSyncInterface $entity */
    $row['id'] = $entity->id();
    $row['type'] = str_replace("_", ' ', $entity->get('type')->value);
    $image = $entity->get('instagram_sync_image')->first()->entity ?? NULL;
    if ($image) {
      $image_style = $this->imageStyleStorage->load('thumbnail');

      $row['image'] = [
        'data' => [
          '#theme' => 'image',
          '#uri' => $image_style->buildUrl($image->getFileUri()),
        ],
      ];
    }
    else {
      $row['image'] = $this->t('No image');
    }

    $row['caption'] = $entity->get('caption')->value ?? 0;
    $row['like_count'] = $entity->get('like_count')->value ?? 0;
    $row['comments_count'] = $entity->get('comments_count')->value ?? 0;
    $row['created'] = $this->dateFormatter->format($entity->get('created')->value);
    $row['changed'] = $this->dateFormatter->format($entity->getChangedTime());
    return $row + parent::buildRow($entity);
  }

}
