<?php

namespace Drupal\instagram_sync\Plugin\APIType;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\instagram_sync\APITypePluginBase;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Plugin implementation for Instagram login.
 *
 * @APIType(
 *   id = "instagram_login",
 *   label = @Translation("Instagram API with Instagram Login"),
 *   description = @Translation("Plugin to use instagram login API."),
 * )
 */
class InstagramLogin extends APITypePluginBase {

  const INSTAGRAM_SYNC_API_URL = 'https://graph.instagram.com/';

  /**
   * Request service.
   *
   * @var \GuzzleHttp\ClientInterface
   *   Request service variable.
   */
  protected ClientInterface $httpClient;

  /**
   * Module list service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   *   Module list service variable.
   */
  protected ModuleExtensionList $moduleList;

  /**
   * Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   *   Logger service variable.
   */
  protected LoggerChannelInterface $logger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('extension.list.module'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $configFactory, ClientInterface $httpClient, ModuleExtensionList $extensionList, LoggerChannelFactoryInterface $loggerChannelFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $configFactory);
    $this->httpClient = $httpClient;
    $this->moduleList = $extensionList;
    $this->logger = $loggerChannelFactory->get('instagram_sync');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $pluginId = $this->config->get('api_type_plugin');
    $form['access_token'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Access Token'),
      '#description' => $this->t('Enter your long-term access token here.'),
      '#required' => TRUE,
      '#default_value' => $this->config->get('plugins.' . $pluginId . '.access_token') ?? '',
    ];

    $form['profile_id'] = [
      '#type' => 'hidden',
      '#default_value' => $this->config->get('plugins.' . $pluginId . '.profile_id') ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface &$form_state) {
    $pluginId = $form_state->getValue('api_type_plugin');
    $pluginValues = $form_state->getValue('plugins')[$pluginId] ?? NULL;
    $accessToken = $pluginValues['access_token'] ?? NULL;
    $profileId = NULL;
    if ($accessToken) {
      try {
        $profileId = $this->getProfileId(['access_token' => $accessToken]);
        $pluginValues['profile_id'] = $profileId;
        $form_state->setValue('plugins', [$pluginId => $pluginValues]);
      }
      catch (\Exception $e) {
        $this->logger->error($e->getMessage(), ['exception' => $e]);
      }
    }
    if (!$profileId) {
      $form_state->setErrorByName("plugins][$pluginId][access_token", $this->t('Please enter valid access token.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function importMedia(array $params = []): object|null {
    $url = $this->getApiUrl() . $this->config->get('plugins.instagram_login.profile_id') . '/media';
    $query = [
      'access_token' => $this->config->get('plugins.instagram_login.access_token'),
      'limit' => INSTAGRAM_SYNC_MAX_QUERY_LIMIT,
      'fields' => implode(',', [
        'id',
        'caption',
        'media_type',
        'media_url',
        'timestamp',
        'thumbnail_url',
        'like_count',
        'comments_count',
        'permalink',
        'children{id,media_type,media_url,timestamp,children,thumbnail_url}',
      ]),
    ];
    $query += $params;
    $options = [
      'query' => $query,
    ];

    try {
      $result = $this->httpClient->request('GET', $url, $options);
      return json_decode($result->getBody()->getContents());
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getApiUrl(): string {
    return self::INSTAGRAM_SYNC_API_URL . 'v21.0/';
  }

  /**
   * {@inheritdoc}
   */
  public function getProfileId(array $data): string|null {
    $url = $this->getApiUrl() . 'me';

    $options = [
      'query' => [
        'access_token' => $data['access_token'],
      ],
    ];

    try {
      $result = $this->httpClient->request('GET', $url, $options);
      if ($result->getStatusCode() === 200) {
        $values = json_decode($result->getBody()->getContents());
        return $values->id ?? '';
      }
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function refreshAccess(array $data): array|null {
    $url = self::INSTAGRAM_SYNC_API_URL . 'refresh_access_token';

    $options = [
      'query' => [
        'grant_type' => 'ig_refresh_token',
        'access_token' => $data['access_token'],
      ],
    ];

    try {
      $result = $this->httpClient->request('GET', $url, $options);
      if ($result->getStatusCode() === 200) {
        $response = json_decode($result->getBody()->getContents(), TRUE);
        if (!empty($response['access_token'])) {
          $this->config->set('plugins.instagram_login.access_token', $response['access_token'])->save();
        }

        return $response;
      }
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }
    return NULL;
  }

}
