<?php

namespace Drupal\instagram_sync\Services;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\instagram_sync\Utility\InstagramAPIMedia;
use Drupal\instagram_sync\Utility\InstagramAPIMediaInterface;
use Drupal\instagram_sync\Utility\InstagramApiRemoteResource;
use Drupal\token\TokenInterface;
use Drupal\instagram_sync\Entity\InstagramSync;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Service description.
 */
class InstagramSyncEntityManager {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   *   Entity type manager service variable.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Token interface service.
   *
   * @var \Drupal\token\TokenInterface
   *   Token interface service variables.
   */
  protected TokenInterface $tokenManager;

  /**
   * File repository service.
   *
   * @var \Drupal\file\FileRepositoryInterface
   *   File repository service variables.
   */
  protected FileRepositoryInterface $fileRepository;

  /**
   * File system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   *   File system service variable.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   *   Logger service variable.
   */
  protected LoggerChannelInterface $logger;

  /**
   * Constructs a new CustomService object.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, TokenInterface $tokenManager, FileRepositoryInterface $fileRepository, FileSystemInterface $fileSystem, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->tokenManager = $tokenManager;
    $this->fileRepository = $fileRepository;
    $this->fileSystem = $fileSystem;
    $this->logger = $loggerChannelFactory->get('instagram_sync');
  }

  /**
   * Transforms instagram API object to a Drupal entity.
   *
   * @param \Drupal\instagram_sync\Utility\InstagramAPIMediaInterface $instagramAPIMedia
   *   Instagram API media object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function createInstagramEntity(InstagramAPIMediaInterface $instagramAPIMedia) {
    $storage = $this->entityTypeManager->getStorage('instagram_sync');

    $ids = $storage->getQuery()
      ->condition('label', $instagramAPIMedia->getId())
      ->range(0, 1)
      ->accessCheck(FALSE)
      ->execute();

    // Load the entity if it exists.
    if (empty($ids)) {
      /** @var \Drupal\instagram_sync\Entity\InstagramSync $entity */
      $entity = $storage->create([
        'label' => $instagramAPIMedia->getId(),
        'type' => $instagramAPIMedia->getMediaType(),
      ]);
    }
    else {
      $id = array_shift($ids);
      $entity = $storage->load($id);
    }
    try {
      $entity->set('permalink', ['uri' => $instagramAPIMedia->getPermalink()]);
      $entity->set('caption', $instagramAPIMedia->getCaption());
      $entity->set('like_count', $instagramAPIMedia->getLikeCount());
      $entity->set('comments_count', $instagramAPIMedia->getCommentsCount());
      $entity->set('created', $instagramAPIMedia->getCreatedTime());
      $this->saveFiles('instagram_sync_image', $entity, $instagramAPIMedia->getImageUrl());
      if ($instagramAPIMedia->getMediaType() === InstagramAPIMedia::TYPE_VIDEO) {
        $this->saveFiles('instagram_sync_video_file', $entity, $instagramAPIMedia->getMediaUrl());
      }
      $entity->save();
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }
  }

  /**
   * Saves media file by URL as a Drupal file.
   *
   * @param string $fieldName
   *   Field to save.
   * @param \Drupal\instagram_sync\Entity\InstagramSync $entity
   *   instagram Entity to process.
   * @param array $resources
   *   Media URLs to save.
   *
   * @return void
   *   Nothing to return.
   */
  protected function saveFiles(string $fieldName, InstagramSync &$entity, array $resources = []) {
    $settings = $entity->getFieldDefinition($fieldName)->getSettings();
    if (!empty($settings)) {
      $scheme = $settings['uri_scheme'] ?? 'public';
      $directory = "{$scheme}://{$settings['file_directory']}/";
      $directory = $this->tokenManager->replace($directory);
      try {
        $this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);
        $entityMedia = [];
        if (!$entity->get($fieldName)->isEmpty()) {
          $entityMedia = $entity->get($fieldName)->getValue();
        }

        if ($entity->isNew() || (count($resources) !== count($entityMedia))) {
          $fileStorage = $this->entityTypeManager->getStorage('file');

          $files = [];
          foreach ($resources as $resource) {
            if ($resource instanceof InstagramApiRemoteResource) {
              $fileName = "{$directory}" . basename($resource->getFileName() . '.' . $resource->getExtension());

              $image_data = file_get_contents($resource->getUrl());
              $file = $this->fileRepository->writeData($image_data, $fileName);
              $file->save();
              $files[] = ['target_id' => $file->id()];
            }
          }
          $entity->set($fieldName, $files);
          $entity->save();

          if (!empty($entityMedia)) {
            $ids = [];
            foreach ($entityMedia as $media) {
              $ids[] = $media['target_id'];
            }
            $files = $fileStorage->loadMultiple($ids);
            foreach ($files as $file) {
              $file->delete();
            }
          }
        }
      }
      catch (\Exception $e) {
        $this->logger->error($e->getMessage(), ['exception' => $e]);
      }
    }
  }

}
