<?php

namespace Drupal\instagram_sync\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\instagram_sync\ApiTypePluginManager;
use Drupal\instagram_sync\Exception\InstagramSyncException;
use Drupal\instagram_sync\Utility\InstagramAPIMedia;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Service description.
 */
class InstagramSyncService {

  /**
   * API type plugin manager service.
   *
   * @var \Drupal\instagram_sync\ApiTypePluginManager
   *   API type plugin manager service variable.
   */
  protected ApiTypePluginManager $apiTypePluginManager;

  /**
   * State service.
   *
   * @var \Drupal\Core\State\StateInterface
   *   State interface service variable.
   */
  protected StateInterface $state;

  /**
   * Config service.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   *   Config service variable.
   */
  protected ImmutableConfig $config;

  /**
   * Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   *   Logger service variable.
   */
  protected LoggerChannelInterface $logger;

  /**
   * Query params needs to be saved for the next query.
   */
  protected array $nextQueryParams = [];

  /**
   * Constructs a new CustomService object.
   */
  public function __construct(ApiTypePluginManager $apiTypePluginManager, StateInterface $state, ConfigFactoryInterface $configFactory, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->apiTypePluginManager = $apiTypePluginManager;
    $this->state = $state;
    $this->config = $configFactory->get(INSTAGRAM_SYNC_CONFIG_NAME);
    $this->logger = $loggerChannelFactory->get('instagram_sync');
  }

  /**
   * Gets refresh token from the instagram API.
   */
  public function refreshAccess(string $pluginId, array $data): array|null {
    if (!$this->apiTypePluginManager->hasDefinition($pluginId)) {
      return NULL;
    }

    /** @var \Drupal\instagram_sync\APITypePluginBase $pluginInstance */
    $pluginInstance = $this->apiTypePluginManager->createInstance($pluginId);
    try {
      $result = $pluginInstance->refreshAccess($data);
      $this->state->set('instagram_sync_last_token_update_run', time());

      return $result;
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }

    return NULL;
  }

  /**
   * Gets media object from the API.
   */
  public function getPosts(string $pluginId, $query_params = []): array {
    $result = [];
    if (!$this->apiTypePluginManager->hasDefinition($pluginId)) {
      return [];
    }

    /** @var \Drupal\instagram_sync\APITypePluginBase $pluginInstance */
    $pluginInstance = $this->apiTypePluginManager->createInstance($pluginId);
    try {
      $data = $pluginInstance->importMedia($query_params);
      if (isset($data->data)) {
        $options = [
          'video_ignore' => !!$this->config->get('settings.video_ignore'),
          'carousel_ignore' => !!$this->config->get('settings.carousel_ignore'),
        ];
        foreach ($data->data as $media) {
          try {
            $result[] = new InstagramAPIMedia($media, $options);
          }
          catch (InstagramSyncException $e) {
            $this->logger->error($e->getMessage(), ['exception' => $e]);
          }
        }
        $this->state->set('instagram_sync_last_import_run', time());
      }

      if (!empty($data->paging->cursors->after)) {
        $this->addNextQueryParam('after', $data->paging->cursors->after);
      }

      return $result;
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage(), ['exception' => $e]);
    }

    return [];
  }

  /**
   * Saves GET params fot the next request.
   */
  protected function addNextQueryParam(string $param, string $value): void {
    $this->nextQueryParams[$param] = $value;
  }

  /**
   * Gets GET params fot the current request.
   */
  public function getNextQueryParams(): array {
    return $this->nextQueryParams;
  }

}
