<?php

namespace Drupal\instagram_sync\Utility;

use Drupal\instagram_sync\Exception\InstagramSyncException;

/**
 * Defines a Fabric class with some methods.
 */
class InstagramAPIMedia implements InstagramAPIMediaInterface {

  const TYPE_VIDEO = 'video';

  const TYPE_CAROUSEL = 'carousel_album';

  const TYPE_IMAGE = 'image';

  /**
   * Instagram Media id.
   */
  protected string $id;

  /**
   * Instagram Media type.
   */
  protected string $mediaType;

  /**
   * Instagram Media images array.
   */
  protected array $imageUrl = [];

  /**
   * Instagram Media media objects array.
   */
  protected array $mediaUrl = [];

  /**
   * Instagram Media caption.
   */
  protected string $caption;

  /**
   * Instagram Media likes count.
   */
  protected int $likeCount;

  /**
   * Instagram Media comments count.
   */
  protected int $commentsCount;

  /**
   * Instagram media permalink.
   */

  protected string $permalink;

  /**
   * Instagram Media created time.
   */
  protected int $timestamp;

  /**
   * Object constructor.
   *
   * @param object $media
   *   Media object values.
   * @param array $options
   *   Options values.
   *
   * @throws \Drupal\instagram_sync\Exception\InstagramSyncException
   */
  public function __construct(object $media, array $options) {
    if (empty($media->media_url)) {
      throw new InstagramSyncException("Media URL doesn't exists. Can't process this media.");
    }

    $video_ignore = $options['video_ignore'] ?? FALSE;
    $carousel_ignore = $options['carousel_ignore'] ?? FALSE;

    $this->id = $media->id;
    $this->mediaType = strtolower($media->media_type);
    $this->timestamp = strtotime($media->timestamp) ?: 0;
    $this->caption = $media->caption ?? '';
    $this->likeCount = $media->like_count ?? 0;
    $this->commentsCount = $media->comments_count ?? 0;
    $this->permalink = $media->permalink ?? '';

    switch ($this->mediaType) {
      case self::TYPE_IMAGE:
        $this->mediaUrl[] = new InstagramApiRemoteResource($media->media_url);
        $this->imageUrl[] = new InstagramApiRemoteResource($media->media_url);
        break;

      case self::TYPE_CAROUSEL:
        $this->mediaUrl[] = new InstagramApiRemoteResource($media->media_url);
        if (!empty($media->children->data)) {
          foreach ($media->children->data as $child) {
            $this->imageUrl[] = new InstagramApiRemoteResource($child->media_url);
            if ($carousel_ignore) {
              break;
            }
          }
        }
        break;

      case self::TYPE_VIDEO:
        if (!$video_ignore) {
          $this->mediaUrl[] = new InstagramApiRemoteResource($media->media_url);
        }
        $this->imageUrl[] = new InstagramApiRemoteResource($media->thumbnail_url);
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->id;
  }

  /**
   * {@inheritdoc}
   */
  public function getMediaType(): string {
    return $this->mediaType;
  }

  /**
   * {@inheritdoc}
   */
  public function getMediaUrl(): array {
    return $this->mediaUrl;
  }

  /**
   * {@inheritdoc}
   */
  public function getImageUrl(): array {
    return $this->imageUrl;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime(): int {
    return $this->timestamp;
  }

  /**
   * {@inheritdoc}
   */
  public function getCaption(): string {
    return $this->caption;
  }

  /**
   * {@inheritdoc}
   */
  public function getLikeCount(): int {
    return $this->likeCount;
  }

  /**
   * {@inheritdoc}
   */
  public function getCommentsCount(): int {
    return $this->commentsCount;
  }

  /**
   * {@inheritdoc}
   */
  public function getPermalink(): string {
    return $this->permalink;
  }

}
